-------------------------------------------------------------------------------
--|> Buttons
-- Manages the butons on the frame that will perform certain functions
-------------------------------------------------------------------------------
--|> UPVALUE GLOBALS
-------------------------------------------------------------------------------
local weizPVP = weizPVP
local UIFrameFadeIn = UIFrameFadeIn
local UIFrameFadeOut = UIFrameFadeOut
local floor = floor
local C_Timer = C_Timer
local C_Timer_After = C_Timer.After

-------------------------------------------------------------------------------
local DragBottomRight
local minimized = false
local MinMaxButtonTexture
local MinMaxButton
local ClearButtonTexture
local ClearButton

-------------------------------------------------------------------------------
-- |> LIBS
-------------------------------------------------------------------------------
local SM = LibStub:GetLibrary("LibSharedMedia-3.0")

-------------------------------------------------------------------------------
--|> LOCAL FUNCTIONS
-------------------------------------------------------------------------------

--=> Minimize Frame
-----------------------------------------------------------------------
local coreHeightPrevious
local function MinFrame()
	coreHeightPrevious = weizPVP.CoreFrame:GetHeight()
	weizPVP.CoreFrame:Hide()
	weizPVP.CoreFrame:SetClampedToScreen(false)
	weizPVP.FooterFrame:Hide()
	weizPVP.FooterFrame:SetClampedToScreen(false)
	DragBottomRight:Hide()
end

--=> Mazimize/Restore Frame
-----------------------------------------------------------------------
local function MaxFrame()
	weizPVP.CoreFrame:SetClampedToScreen(true)
	weizPVP.CoreFrame:SetHeight(coreHeightPrevious)
	weizPVP.CoreFrame:Show()
	weizPVP.FooterFrame:SetClampedToScreen(true)
	weizPVP.FooterFrame:Show()
	DragBottomRight:Show()
end

--=> Snap to Bars
-----------------------------------------------------------------------
local frameHeight
local row
local rawCalc
local barsInDisplay
local function SnapHeightToBars()
	if not InCombatLockdown() then
		frameHeight = weizPVP.CoreFrame:GetHeight()
		row = weizPVP.Options.Bars.RowHeight + weizPVP.Options.Bars.VerticalSpacing
		rawCalc = frameHeight / row
		barsInDisplay = floor(frameHeight / row)
		if rawCalc - barsInDisplay >= .5 then
			barsInDisplay = barsInDisplay + 1
		end
		if barsInDisplay < 1 then
			barsInDisplay = 1
		end

		weizPVP.CoreFrame:SetHeight(barsInDisplay * row)
		weizPVP:SaveCoreFramePosition()
		weizPVP:AlignFrameToPixels(weizPVP.CoreFrame)
	end
end

--=> Drag Bottom Right
-------------------------------------------------------------------------------
local lastTime = GetTime()
local footerToSide
local footerToBottom
local lastTry = 0
local timeBetweenChatMessage = 5
function weizPVP:CreateResizeGrip()
	DragBottomRight = CreateFrame("Button", "weizPVPResizeGripBottomRight", weizPVP.FooterFrame)
	DragBottomRight:SetFrameLevel(99)
	DragBottomRight:SetFrameStrata("MEDIUM")
	DragBottomRight:SetNormalTexture("Interface\\AddOns\\weizPVP\\Icons\\resizeGrip.tga")
	DragBottomRight:SetHighlightTexture("Interface\\AddOns\\weizPVP\\Icons\\resizeGrip.tga")
	DragBottomRight:GetHighlightTexture():SetVertexColor(1, 0.8, 0, 1)
	DragBottomRight:SetWidth(16)
	DragBottomRight:SetHeight(16)
	DragBottomRight:SetScale(1)
	DragBottomRight:SetPoint("BOTTOMRIGHT", weizPVP.FooterFrame, "BOTTOMRIGHT", -1, 1)
	DragBottomRight:Show()

	--> DRAG RESIZE: OnMouseDown
	-----------------------------------
	DragBottomRight:SetScript(
		"OnMouseDown",
		function(self, button)
			if weizPVP.CoreFrame.isLocked == false and button == "LeftButton" then
				weizPVP.CoreFrame.isResizing = true
				weizPVP.CoreFrame:StopMovingOrSizing("TOP")
				weizPVP.CoreFrame:StopMovingOrSizing("LEFT")
				weizPVP.CoreFrame:StartSizing("BOTTOMRIGHT")
				footerToSide = weizPVP.CoreFrame:GetWidth() + (UIParent:GetRight() - weizPVP.FooterFrame:GetRight())
				footerToBottom = weizPVP.CoreFrame:GetHeight() + (UIParent:GetBottom() + weizPVP.FooterFrame:GetBottom())
				if footerToSide > 500 then
					footerToSide = 500
				end
				if footerToBottom > weizPVP.Options.Frames.List.Height then
					footerToBottom = weizPVP.Options.Frames.List.Height
				end
				weizPVP.CoreFrame:SetMaxResize(footerToSide, footerToBottom)
			elseif weizPVP.CoreFrame.isLocked == true then
				weizPVP:SetStatusBarMessage(" Window is |cffff3838LOCKED|r", true, 1)

				weizPVP.PlaySoundAlert(SM:Fetch("sound", "weizPVP: Error 1"))
				if lastTry + timeBetweenChatMessage < GetTime() then
					weizPVP:PrintAddonMessage("Window is |cffff3838LOCKED|r. Use Ctrl + Right click on the title bar to unlock it.")
					lastTry = GetTime()
				end
			end
		end
	)

	--> DRAG RESIZE: OnMouseUp
	-----------------------------------
	DragBottomRight:SetScript(
		"OnMouseUp",
		function(self, button)
			if weizPVP.CoreFrame.isLocked == false and button == "LeftButton" then
				SnapHeightToBars()
				weizPVP.CoreFrame:StopMovingOrSizing()
				weizPVP.CoreFrame.isResizing = false
				weizPVP.HeaderFrame:SetMovable(true)
				if weizPVP.CoreFrame.isResizing == true and (button == "LeftButton") then
					weizPVP.CoreFrame:StopMovingOrSizing()
					weizPVP.CoreFrame.isResizing = false
				end
				weizPVP:RefreshCurrentList()
				weizPVP:SaveCoreFramePosition()
			end
		end
	)
	--> DRAG RESIZE: OnUpdate
	-----------------------------------
	DragBottomRight:SetScript(
		"OnUpdate",
		function(self)
			if weizPVP.CoreFrame.isResizing and GetTime() + 0.3 > lastTime then
				weizPVP:RefreshCurrentList()
				lastTime = GetTime()
			end
		end
	)
end

--=> Show Arrow
-------------------------------------------------------------------------------
local function ShowArrow()
	if minimized then
		MinMaxButtonTexture:GetNormalTexture():SetRotation(3.14159)
	else
		MinMaxButtonTexture:GetNormalTexture():SetRotation(0)
	end
	UIFrameFadeIn(MinMaxButtonTexture, 0.2, 0, 1)
	MinMaxButtonTexture:Show()
end

--=> Create Header Icons
-------------------------------------------------------------------------------
local ag
local anim
function weizPVP:CreateHeaderIcons()
	--> MinMaxButtonTexture
	-------------------------------------------------------------------------------
	MinMaxButtonTexture = CreateFrame("Button", "weizPVP.HeaderFrame.MinMaxButtonTexture", weizPVP.HeaderFrame)
	MinMaxButtonTexture:SetPoint("RIGHT", weizPVP.HeaderFrame, "RIGHT", 0, 0)
	MinMaxButtonTexture:SetSize(32, 32)
	MinMaxButtonTexture:SetNormalTexture("Interface\\addons\\weizPVP\\Icons\\arrow.tga")
	MinMaxButtonTexture:GetNormalTexture():SetVertexColor(0.8, 0.8, 0.8, 1)

	MinMaxButton = CreateFrame("Button", "weizPVP.HeaderFrame.MinMaxButton", MinMaxButtonTexture)
	MinMaxButton:SetPoint("CENTER", MinMaxButtonTexture, "CENTER")
	MinMaxButton:SetSize(24, 24)

	--> MIN/MAX BUTTON: OnMouseDown
	-----------------------------------
	MinMaxButton:SetScript(
		"OnMouseDown",
		function(self, event)
			if not InCombatLockdown() then --!! added until we fix frames
				if event == "LeftButton" then
					--> MINIZING
					if minimized == false then
						UIFrameFadeOut(weizPVP.CoreFrame, 0.1, 1, 0)
						UIFrameFadeOut(MinMaxButtonTexture, 0.1, 1, 0.5)
						UIFrameFadeOut(weizPVP.FooterFrame, 0.1, 1, 0)
						minimized = true
						C_Timer_After(0.09, ShowArrow)
						C_Timer_After(0.1, MinFrame)
						weizPVP.CoreFrame:GetHeight(coreHeightPrevious)
						weizPVP.CoreFrame:Hide()
						weizPVP.FooterFrame:Hide()
						weizPVP.OuterIconsFrame:Hide()
					else
						--> RESTORING
						UIFrameFadeIn(weizPVP.CoreFrame, 0.1, 0, 1)
						UIFrameFadeOut(MinMaxButtonTexture, 0.1, 1, 0.5)
						UIFrameFadeIn(weizPVP.FooterFrame, 0.1, 0, 1)
						minimized = false
						C_Timer_After(0.09, ShowArrow)
						C_Timer_After(0.1, MaxFrame)
						weizPVP.CoreFrame:SetHeight(coreHeightPrevious)
						weizPVP.CoreFrame:Show()
						weizPVP.FooterFrame:Show()
						weizPVP.CoreFrame:SetHeight(coreHeightPrevious)
						weizPVP.FooterFrame:SetHeight(weizPVP.Options.Frames.Footer.Height)
						DragBottomRight:Show()
						weizPVP.OuterIconsFrame:Show()
					end
				end
			end
		end
	)
	--> MIN/MAX BUTTON: OnEnter
	-----------------------------------
	MinMaxButton:SetScript(
		"OnEnter",
		function(self, event)
			MinMaxButtonTexture:SetNormalTexture("Interface\\addons\\weizPVP\\Icons\\arrow.tga")
			MinMaxButtonTexture:GetNormalTexture():SetVertexColor(1, 1, 1, 1)
		end
	)
	--> MIN/MAX BUTTON: OnLeave
	-----------------------------------
	MinMaxButton:SetScript(
		"OnLeave",
		function(self, event)
			MinMaxButtonTexture:SetNormalTexture("Interface\\addons\\weizPVP\\Icons\\arrow.tga")
			MinMaxButtonTexture:GetNormalTexture():SetVertexColor(0.8, 0.8, 0.8, 1)
		end
	)

	--> Clear List Button
	-------------------------------------------------------------------------------
	ClearButtonTexture = CreateFrame("Button", "weizPVP.ClearButtonTexture", weizPVP.HeaderFrame)
	ClearButtonTexture:SetPoint("RIGHT", MinMaxButton, "LEFT", 2, 0)
	ClearButtonTexture:SetSize(32, 32)
	ClearButtonTexture:SetAlpha(1)
	ClearButtonTexture:SetNormalTexture("Interface\\addons\\weizPVP\\Icons\\button_refresh.tga")
	ClearButtonTexture:GetNormalTexture():SetVertexColor(0.9, 0.6, 0)

	ClearButton = CreateFrame("Button", "weizPVP.HeaderFrame.ClearButton", ClearButtonTexture)
	ClearButton:SetPoint("CENTER", ClearButtonTexture, "CENTER")
	ClearButton:SetSize(24, 24)

	--> Rotation Animation
	ag = ClearButtonTexture:CreateAnimationGroup()
	anim = ag:CreateAnimation("Rotation")
	anim:SetDegrees(-360)
	anim:SetDuration(0.4)

	--> CLEAR: OnMouseDown
	-----------------------------------
	ClearButton:SetScript(
		"OnMouseDown",
		function(self, event)
			if event == "LeftButton" then
				weizPVP:ClearListData()
				ag:Play()
			end
		end
	)
	--> CLEAR: OnEnter
	-----------------------------------
	ClearButton:SetScript(
		"OnEnter",
		function(self, event)
			ClearButtonTexture:GetNormalTexture():SetVertexColor(1, 0.7, 0)
		end
	)
	--> CLEAR: OnLeave
	-----------------------------------
	ClearButton:SetScript(
		"OnLeave",
		function(self, event)
			ClearButtonTexture:GetNormalTexture():SetVertexColor(0.9, 0.6, 0)
		end
	)

	weizPVP.HeaderFrame.MinMaxButton = MinMaxButton
end
