-------------------------------------------------------------------------------
--|> StealthAlert
-- A popup frame dedicated to stealth detection
-------------------------------------------------------------------------------
--|> UPVALUE GLOBALS
-------------------------------------------------------------------------------
local weizPVP = weizPVP
local C_Timer_After = C_Timer.After
------------------------------------------------------------------------------
--|> LIBS
-------------------------------------------------------------------------------
local SM = LibStub:GetLibrary("LibSharedMedia-3.0")
local fade1
local fade2

-------------------------------------------------------------------------------
--|> BUILD FRAME
-------------------------------------------------------------------------------
local topOffset
function weizPVP:CreateStealthAlertFrame()
	--> Get a position 1/3rd from top of screen
	topOffset = UIParent:GetTop() / 3

	--> Create Alert Frame
	weizPVP.StealthAlertFrame = CreateFrame("Frame", "weizPVP.StealthAlertFrame", UIParent)
	weizPVP.StealthAlertFrame:SetFrameStrata("HIGH")
	weizPVP.StealthAlertFrame:SetPoint("TOP", UIParent, "CENTER", 0, topOffset + 60)
	weizPVP.StealthAlertFrame:SetHeight(90)
	weizPVP.StealthAlertFrame:SetWidth(160)
	weizPVP.StealthAlertFrame:SetBackdrop(
		{
			bgFile = SM:Fetch("background", "weizPVP: Background"),
			tile = false,
			edgeFile = SM:Fetch("border", "weizPVP: Border"),
			edgeSize = 2,
			insets = {left = 0, right = 0, top = 0, bottom = 0}
		}
	)
	weizPVP.StealthAlertFrame:SetBackdropColor(0, 0, 0, 0.6)
	weizPVP.StealthAlertFrame:SetBackdropBorderColor(0, 0, 0, 0.8)
	weizPVP.StealthAlertFrame:SetFrameLevel(2)
	weizPVP.StealthAlertFrame:Hide()

	--=> StealthAlertFrame.EventText
	-----------------------------------------------------------
	weizPVP.StealthAlertFrame.EventText = weizPVP.StealthAlertFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	weizPVP.StealthAlertFrame.EventText:SetPoint("TOP", weizPVP.StealthAlertFrame, "TOP", 0, -8)
	weizPVP.StealthAlertFrame.EventText:SetFont(SM:Fetch("font", "Roboto Condensed BoldItalic"), 20, "OUTLINE")
	weizPVP.StealthAlertFrame.EventText:SetJustifyH("CENTER")
	weizPVP.StealthAlertFrame.EventText:SetTextColor(1, 0.3, 0.9, 1)
	weizPVP.StealthAlertFrame.EventText:SetText("STEALTH DETECTED")
	weizPVP.StealthAlertFrame.EventText:Hide()

	--=> StealthAlertFrame.PlayerNameText
	-----------------------------------------------------------
	weizPVP.StealthAlertFrame.PlayerNameText = weizPVP.StealthAlertFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	weizPVP.StealthAlertFrame.PlayerNameText:SetPoint("TOP", weizPVP.StealthAlertFrame.EventText, "BOTTOM", 0, -4)
	weizPVP.StealthAlertFrame.PlayerNameText:SetFont(SM:Fetch("font", "Roboto Condensed Bold"), 20, "OUTLINE")
	weizPVP.StealthAlertFrame.PlayerNameText:SetJustifyH("CENTER")
	weizPVP.StealthAlertFrame.PlayerNameText:SetTextColor(1, 1, 1, 1)
	weizPVP.StealthAlertFrame.PlayerNameText:SetText("PLAYERNAME")
	weizPVP.StealthAlertFrame.PlayerNameText:Hide()

	--> Animation
	weizPVP.StealthAlertFrame.anim = weizPVP.StealthAlertFrame:CreateAnimationGroup()
	weizPVP.StealthAlertFrame.anim:SetScript(
		"OnFinished",
		function()
			weizPVP.StealthAlertFrame:Hide()
		end
	)
	fade1 = weizPVP.StealthAlertFrame.anim:CreateAnimation("Alpha")
	fade1:SetDuration(0.1)
	fade1:SetFromAlpha(0)
	fade1:SetToAlpha(1)
	fade1:SetEndDelay(2)
	--> Set the number of seconds the animation delays after finishing.
	fade1:SetOrder(1)

	fade2 = weizPVP.StealthAlertFrame.anim:CreateAnimation("Alpha")
	fade2:SetDuration(0.2)
	fade2:SetFromAlpha(1)
	fade2:SetToAlpha(0)
	fade2:SetOrder(2)
end

--=> Stealth Alert
-----------------------------------------------------------
local playSound = true
local function EnableSound()
	playSound = true
end

--!! TEST VIA CHAT: /run weizPVP:ShowStealthAlert("STEALTH DETECTED", "Playername", 1,1,1)
--> Show Alert
local frameWidth
local frameHeight
function weizPVP:ShowStealthAlert(EventText, PlayerNameText, r, g, b)
	if weizPVP.Options.StealthAlert.Enabled == true then
		if weizPVP.StealthAlertFrame.anim:IsPlaying() then
			weizPVP.StealthAlertFrame.anim:Stop()
		end

		weizPVP.StealthAlertFrame.EventText:SetText(EventText)
		weizPVP.StealthAlertFrame.PlayerNameText:SetText(PlayerNameText)
		weizPVP.StealthAlertFrame.PlayerNameText:SetTextColor(r, g, b)

		--> Set Background to resize based on text boundries
		frameHeight = weizPVP.StealthAlertFrame:GetTop() - weizPVP.StealthAlertFrame.PlayerNameText:GetBottom() + 8
		weizPVP.StealthAlertFrame:SetHeight(frameHeight)

		if weizPVP.StealthAlertFrame.EventText:GetWidth() > weizPVP.StealthAlertFrame.PlayerNameText:GetWidth() then
			frameWidth = weizPVP.StealthAlertFrame.EventText:GetWidth()
		else
			frameWidth = weizPVP.StealthAlertFrame.PlayerNameText:GetWidth()
		end

		weizPVP.StealthAlertFrame:SetWidth(frameWidth + 48)

		--> Show Elements
		weizPVP.StealthAlertFrame.EventText:Show()
		weizPVP.StealthAlertFrame.PlayerNameText:Show()
		weizPVP.StealthAlertFrame:SetAlpha(0)
		weizPVP.StealthAlertFrame:Show()
		weizPVP.StealthAlertFrame.anim:Play()
		--> play stealth alert sound
		if playSound and weizPVP.Options.StealthAlert.EnableSound == true then
			PlaySoundFile("Interface\\AddOns\\weizPVP\\Sounds\\alert-1.ogg")
			playSound = false
			C_Timer_After(3, EnableSound)
		end
	end
end
